/*
 *	File:		DILette.c
 * 
 *	Contains:	Minimal demonstration application for the CDILs
 *
 *	Written by:	David Fedor, Newton Developer Technical Support
 * 
 *	Copyright:	 1996 by Apple Computer, Inc.  All rights reserved.
 *
 *  Notes:
 *     This is intended to show the minimum necessary to write an application
 *     which uses the DILs.  It is not a "friendly" application and has no
 *     UI whatsoever... the better to provide a simple demonstration!
 *
 *     DILette is intended to be used with the Newton package "SoupDrink".
 *     When run, DILette will make a connection and send down the string
 *     "HELO".  It will expect the Newton side to respond with "Hello".
 *     When this response has been received, DILette will disconnect.
 *
 *     If any errors are encountered on this side, a message will be displayed
 *     in MacsBug, and the pipe will be disconnected.
 *
 *     You may incorporate this sample code into your applications without
 *     restriction.  This sample code has been provided "AS IS" and the
 *     responsibility for its operation is 100% yours.  You are not
 *     permitted to modify and redistribute the source as "DTS Sample Code."
 *     If you are going to re-distribute the source, we require that you
 *     make it clear in the source that the code was descended from
 *     Apple-provided sample code, but that you've made changes.
 */ 

// standard MacOS include files
#include <quickdraw.h>
#include <fonts.h>
#include <windows.h>
#include <memory.h>
#include <string.h>

#define forMac  // this is a Macintosh-only source file... DILCPipe.h needs to know.
#include "DILCPipe.h"

// the generic catch-all error handler, which alerts the user, disconnects and returns.
#define HANDLEERROR(err, msg) if (err) { DebugStr(msg); if (ourPipe) { CDPipeDisconnect(ourPipe); CDDisposeCDILObject(ourPipe); } return; }

void main(void)
{
	CommErr		fErr;
	CDILPipe	*ourPipe=0;
	long		length;
	Boolean		eom;
	char		response[100];

    // Do the standard Macintosh initializations
    MaxApplZone();
    InitGraf((Ptr)&qd.thePort);
    InitFonts();
    InitWindows();
    InitCursor();
	
	// initialize the CDIL library
	fErr = CDInitCDIL() ; 
	HANDLEERROR(fErr, "\pError initializing CDILs");

	// create and then initialize the pipe.
	ourPipe = CDCreateCDILObject();
	HANDLEERROR(ourPipe == 0, "\pError creating the pipe")

	// uncomment one of the following three lines, depending on which serial port should be used.
	#define WHICHPORT "Modem Port"
	//#define WHICHPORT "Printer Port"
	//#define WHICHPORT "Printer-Modem Port"

	// only uncomment one of the following lines, depending on what connection type you want to use.	
	// MNP serial
	fErr = (short) CDPipeInit(ourPipe, "Modem","","ModemType \"Newton Serial Connection\" dataBits 8 Parity None Baud 38400 Port ", WHICHPORT, kDefaultBufferSize, kDefaultBufferSize);
	// straight serial
	// fErr = (short) CDPipeInit(ourPipe, "Serial","","Baud 38400 dataBits 8 Parity None Port ", WHICHPORT, kDefaultBufferSize, kDefaultBufferSize);
	// AppleTalk (ADSP)
	// fErr = (short) CDPipeInit(ourPipe, "ADSP", "My DILette Mac", "LocalADSPType \"SoupDrink\" RegisterName 1 LocalADSPName ", 0, kDefaultBufferSize, kDefaultBufferSize);

	HANDLEERROR(fErr, "\pError initializing the pipe");

	// listen for the Newton device's connection
	fErr = CDPipeListen(ourPipe, kDefaultTimeout, 0, 0);
	HANDLEERROR(fErr, "\pError calling CDPipeListen... check the cables... is the Newton application running?");

	// accept the connection from the Newton device
	fErr = CDPipeAccept(ourPipe);
	HANDLEERROR(fErr, "\pError calling CDPipeAccept");

	// now send a string down the pipe!
	length = 5;		// # of characters to transmit
	fErr = CDPipeWrite(ourPipe, "HELO\4", &length, true,0,0,1000,0,0);
	HANDLEERROR(fErr, "\pError calling CDPipeWrite");
	
	// and read the response.
	length = 5;		// # of characters to read
	fErr = CDPipeRead(ourPipe, response, &length, &eom,0,0,1000,0,0);
	HANDLEERROR(fErr, "\pError calling CDPipeRead");
	
	// see if we got the expected response, just for the heck of it.
	response[5] = 0;
	if (strcmp(response, "Hello") != 0)
		HANDLEERROR(1, "\pUnexpected response from SoupDrink (it didn't respond 'Hello')");

	// At this point, we're in business, talking back and forth with the Newton
	// device.  We could do anything at this point, including use the FDIL calls
	// (FDput, etc) to send frames back and forth.  However, we'll just shut down
	// the connection and clean up... we're done!
	CDPipeDisconnect(ourPipe);
	CDDisposeCDILObject(ourPipe);
	CDDisposeCDIL();
	
	// beep for joy and happiness that it all worked as expected.
	SysBeep(500); SysBeep(500);
}